
/* Chrysalide - Outil d'analyse de fichiers binaires
 * loaded.c - prototypes pour l'équivalent Python du fichier "analysis/loaded.c"
 *
 * Copyright (C) 2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "loaded.h"


#include <assert.h>
#include <pygobject.h>


#include <i18n.h>


#include <analysis/loaded.h>



/* Lance l'analyse propre à l'élément chargé. */
static PyObject *py_loaded_content_analyze(PyObject *, PyObject *);

/* Lance l'analyse de l'élément chargé et attend sa conclusion. */
static PyObject *py_loaded_content_analyze_and_wait(PyObject *, PyObject *);

/* Détermine le nombre de vues disponibles pour un contenu. */
static PyObject *py_loaded_content_count_views(PyObject *, PyObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self = contenu binaire à manipuler.                          *
*                args = non utilisé ici.                                      *
*                                                                             *
*  Description : Lance l'analyse propre à l'élément chargé.                   *
*                                                                             *
*  Retour      : Rien (None).                                                 *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_loaded_content_analyze(PyObject *self, PyObject *args)
{
    GLoadedContent *content;                /* Version GLib de l'élément   */

    content = G_LOADED_CONTENT(pygobject_get(self));

    g_loaded_content_analyze(content);

    Py_RETURN_NONE;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self = contenu binaire à manipuler.                          *
*                args = non utilisé ici.                                      *
*                                                                             *
*  Description : Lance l'analyse de l'élément chargé et attend sa conclusion. *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_loaded_content_analyze_and_wait(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Bilan à retourner           */
    GLoadedContent *content;                /* Version GLib de l'élément   */
    bool status;                            /* Bilan de l'opération        */

    content = G_LOADED_CONTENT(pygobject_get(self));

    status = g_loaded_content_analyze_and_wait(content);

    result = status ? Py_True : Py_False;
    Py_INCREF(result);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self = contenu chargé à manipuler.                           *
*                args = non utilisé ici.                                      *
*                                                                             *
*  Description : Détermine le nombre de vues disponibles pour un contenu.     *
*                                                                             *
*  Retour      : Quantité strictement positive.                               *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_loaded_content_count_views(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Instance à retourner        */
    GLoadedContent *content;                /* Version GLib de l'élément   */
    size_t count;                           /* Quantité à retourner        */

    content = G_LOADED_CONTENT(pygobject_get(self));

    count = g_loaded_content_count_views(content);

    result = PyLong_FromUnsignedLongLong(count);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_loaded_content_type(void)
{
    static PyMethodDef py_loaded_content_methods[] = {
        {
            "analyze", py_loaded_content_analyze,
            METH_NOARGS,
            "analyze($self, /)\n--\n\nStart the analysis of the loaded binary and " \
            "send a \"disassembly-done\" signal when done."
        },
        {
            "analyze_and_wait", py_loaded_content_analyze_and_wait,
            METH_NOARGS,
            "analyze_and_wait($self, /)\n--\n\nRun the analysis of the loaded binary and " \
            "wait for its completion."
        },
        {
            "count_views", py_loaded_content_count_views,
            METH_NOARGS,
            "count_views($self, /)\n--\n\nCompute the quantity of available views."
        },
        { NULL }
    };

    static PyGetSetDef py_loaded_content_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_loaded_content_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.analysis.LoadedContent",
        .tp_basicsize   = sizeof(PyObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,

        .tp_doc         = "PyChrysalide loaded content",

        .tp_methods     = py_loaded_content_methods,
        .tp_getset      = py_loaded_content_getseters

    };

    return &py_loaded_content_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.....LoadedContent'.    *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool register_python_loaded_content(PyObject *module)
{
    PyTypeObject *py_loaded_content_type;   /* Type Python 'LoadedContent' */
    PyObject *dict;                         /* Dictionnaire du module      */

    py_loaded_content_type = get_python_loaded_content_type();

    dict = PyModule_GetDict(module);
    pyg_register_interface(dict, "LoadedContent", G_TYPE_LOADED_CONTENT, py_loaded_content_type);

    return true;

}
